
#### functions facilitating post-processing of MCMCglmm model lists
postburnin.MCMCglmm = function(x, burnin= 0) {
	require("coda")
	START = start(x$Sol)
	END = end(x$Sol)
	THIN = thin(x$Sol)
	if(burnin <= START | burnin >= END) stop( paste0("new 'burnin' needs to be > ", START," and < ", END )) 
	if(!(burnin/THIN)%%1 == 0) stop(paste0("new 'burnin' needs to be a multiple of ", THIN, " (thin)"))
	# translate to sequence
	burnin.start = if(burnin == "burnin") (START-1)/THIN +1 else burnin/THIN - (START-1)/THIN  +1
	if(burnin.start < dim(x$VCV)[1]) {
		OldNames = colnames(x$Sol)
	if(dim(x$Sol)[2] > 1) 
		{
		if(burnin != "burnin") x$Sol= mcmc(x$Sol[ burnin.start:nrow(x$Sol),], start= burnin+1, end= END, thin= THIN)
		} else { 
		if(burnin != "burnin") x$Sol= mcmc( matrix(x$Sol[ burnin.start:nrow(x$Sol)], length(x$Sol[ burnin.start:nrow(x$Sol)]), 1), start= burnin+1, end= END, thin= THIN)
		}
	colnames(x$Sol) = OldNames
	if(burnin != "burnin") x$VCV= mcmc(x$VCV[ burnin.start:nrow(x$VCV),], start= burnin+1, end= END, thin= THIN )
	print(paste0("old burnin = ", START-1, "; new burnin = ", start(x$Sol)-1)) 
	} else print("burnin larger than max number of iterations")
return(x)}

postthin.MCMCglmm = function(x, thin= 0) {
	require("coda")
	START = start(x$Sol)
	END = end(x$Sol)
	THIN = thin(x$Sol)
	if(thin < THIN) stop(paste0("new 'thin' needs to be larger than ", THIN, " (thin)"))
	if(!(thin/THIN)%%1 == 0 ) stop(paste0("new 'thin' needs to be a multiple of ", THIN, " (thin)"))
	# translate to sequence
	OldNames = colnames(x$Sol)
	thin = if(thin == "thin") THIN else thin/THIN
		if(dim(x$Sol)[2] > 1) 
		{
		x$Sol = mcmc(x$Sol[seq(1, nrow(x$Sol), by= thin),],	start= START, end= END, thin= thin*THIN)
		} else { 
		x$Sol = mcmc( matrix(x$Sol[seq(1, nrow(x$Sol), by= thin)], length(x$Sol[seq(1, nrow(x$Sol), by= thin)]), 1), start= START, end= END, thin= thin*THIN)
		}
		colnames(x$Sol) = OldNames
		x$VCV = mcmc(x$VCV[seq(1, nrow(x$VCV), by= thin),],	start= START, end= END, thin= thin*THIN)
	print(paste0("old thin = ", THIN, "new thin = ", thin*THIN))
return(x)}

#### functions to extract VCV-derived parameters from MCMCglmm models (per model type)
# 1) UV + GxE model for MIG response
RUN.model.covar.res_SMO = function(model.list= NULL, model.name.list= NULL, digits= 3 ) {
	require("coda")
	Temperature.vec = c("Cold","Warm")
	"model.covar.res" = NULL
	"model.contrast.res" = NULL
	for(i in 1:length(model.name.list))
	{
	model = model.list[[i]]
	model.name = model.name.list[[i]]
	# h2 SMO CONTRAST between temperatures
	SELECTION = colnames(model$VCV[,
		which(
		!grepl("traitLEN",colnames(model$VCV)))])
	SELECTION
	h2.TEMP.CONTRAST = NULL
	h2.TEMP.CONTRAST.res = NULL
	h2.TEMP.CONTRAST = model$VCV[,SELECTION[3]] / (model$VCV[,SELECTION[3]] + model$VCV[,SELECTION[1]] + model$VCV[,SELECTION[7]] + 1) - model$VCV[,SELECTION[6]] / (model$VCV[,SELECTION[6]] + model$VCV[,SELECTION[2]] + model$VCV[,SELECTION[8]] + 1)
	h2.TEMP.CONTRAST.res = round(data.frame(mean = mean(h2.TEMP.CONTRAST), HPDinterval(h2.TEMP.CONTRAST)), digits)
	h2.TEMP.CONTRAST.res$Parameter = "h2.SMO - h2.SMO"
	h2.TEMP.CONTRAST.res$Temperature = "Cold-Warm"
	h2.TEMP.CONTRAST.res$model.name = model.name
	model.covar.res = rbind(model.covar.res, h2.TEMP.CONTRAST.res)
	model.covar.res
	# RG between SMO Cold vs SMO in Warm
	SELECTION = colnames(model$VCV[,
		which(
		grepl("DNA.ID.at.Tagging",colnames(model$VCV)) & 
		!grepl("traitLEN",colnames(model$VCV))
		)])
	SELECTION
	RG.GxE.SMO = NULL
	RG.GxE.SMO.res = NULL
	RG.GxE.SMO = model$VCV[,SELECTION[2]] / sqrt(model$VCV[,SELECTION[1]] * model$VCV[,SELECTION[4]])
	RG.GxE.SMO.res = round(data.frame(mean = mean(RG.GxE.SMO), HPDinterval(RG.GxE.SMO)), digits)
	RG.GxE.SMO.res$Parameter ="RG.SMO.SMO"
	RG.GxE.SMO.res$Temperature = "Warm-Cold"
	RG.GxE.SMO.res$model.name = model.name
	model.covar.res = rbind(model.covar.res, RG.GxE.SMO.res)
	model.covar.res
# temperature-specific parameters:	
	Temperature.vec = c("Cold","Warm") # COLD FIRST!
	for(j in 1:length(Temperature.vec))
	{
	# h2 SMO PER TEMP
	SELECTION = colnames(model$VCV[,
		which(
		!grepl("traitLEN.P2",colnames(model$VCV)) & 
		!grepl(Temperature.vec[Temperature.vec!=Temperature.vec[j]],colnames(model$VCV)))])
	SELECTION
	h2.SMO = NULL
	h2.SMO.res = NULL
	h2.SMO = model$VCV[,SELECTION[2]] / (model$VCV[,SELECTION[2]] + model$VCV[,SELECTION[1]] + model$VCV[,SELECTION[3]] + 1)
	h2.SMO.res = round(data.frame(mean = mean(h2.SMO), HPDinterval(h2.SMO)), digits)
	h2.SMO.res$Parameter ="h2.SMO"
	h2.SMO.res$Temperature = Temperature.vec[j]
	h2.SMO.res$model.name = model.name
	model.covar.res = rbind(model.covar.res, h2.SMO.res)
	model.covar.res	
	# c2 SMO PER TEMP
	SELECTION = colnames(model$VCV[,
		which(
		!grepl("traitLEN.P2",colnames(model$VCV)) &
		!grepl(Temperature.vec[Temperature.vec!=Temperature.vec[j]],colnames(model$VCV)))])
	SELECTION
	c2.SMO = NULL
	c2.SMO.res = NULL
	c2.SMO = model$VCV[,SELECTION[1]] / (model$VCV[,SELECTION[1]] + model$VCV[,SELECTION[2]] + model$VCV[,SELECTION[3]] + 1)
	c2.SMO.res = round(data.frame(mean = mean(c2.SMO),HPDinterval(c2.SMO)), digits)
	c2.SMO.res$Parameter ="c2.SMO"
	c2.SMO.res$Temperature = Temperature.vec[j]
	c2.SMO.res$model.name = model.name
	model.covar.res = rbind(model.covar.res, c2.SMO.res)
	model.covar.res
	} } 
rownames(model.covar.res) = NULL
model.covar.res
}

# 2) BV model, no GxE for MIG,LEN responses per temperature
RUN.model.covar.res_BIV.NoGxE = function(model.list= NULL, model.name.list= NULL, digits = 3 ) {
	require("coda")
	model.covar.res = NULL
	for(i in 1:length(model.name.list))
	{
	model = model.list[[i]]
	model.name = model.name.list[[i]]
	Temperature.vec = c("Cold","Warm") 
	j = if(grepl("Cold",model.name.list[[i]])) 1 else if(grepl("Warm",model.name.list[[i]])) 2 else stop("no temperature indicated in model name")
	# h2 LEN PER TEMP
	SELECTION = colnames(model$VCV[,
		which(
		!grepl("SMO",colnames(model$VCV)))])
	SELECTION
	h2.LEN = NULL
	h2.LEN.res = NULL
	h2.LEN = model$VCV[,SELECTION[2]] / (model$VCV[,SELECTION[1]] + model$VCV[,SELECTION[2]] + model$VCV[,SELECTION[3]] )
	h2.LEN.res = round( data.frame(mean = mean(h2.LEN), HPDinterval(h2.LEN)), digits)
	h2.LEN.res$Parameter = "h2.LEN"
	h2.LEN.res$Temperature = Temperature.vec[j]
	h2.LEN.res$model.name = model.name
	model.covar.res = rbind(model.covar.res, h2.LEN.res)
	model.covar.res
	# c2 LEN PER TEMP
	c2.LEN = NULL
	c2.LEN.res = NULL
	c2.LEN = model$VCV[,SELECTION[1]] / (model$VCV[,SELECTION[1]] + model$VCV[,SELECTION[2]] + model$VCV[,SELECTION[3]])
	c2.LEN.res = round( data.frame(mean = mean(c2.LEN), HPDinterval(c2.LEN)), digits)
	c2.LEN.res$Parameter ="c2.LEN"
	c2.LEN.res$Temperature = Temperature.vec[j]
	c2.LEN.res$model.name = model.name
	model.covar.res = rbind(model.covar.res, c2.LEN.res)
	model.covar.res
 	# h2 SMO PER TEMP
	SELECTION = colnames(model$VCV[,
		which(
		!grepl("LEN.P2",colnames(model$VCV)))])
	SELECTION
	h2.SMO = NULL
	h2.SMO.res = NULL
	h2.SMO = model$VCV[,SELECTION[2]] / (model$VCV[,SELECTION[1]] + model$VCV[,SELECTION[2]] + model$VCV[,SELECTION[3]] + 1)
	h2.SMO.res = round( data.frame(mean = mean(h2.SMO), HPDinterval(h2.SMO)), digits)
	h2.SMO.res$Parameter = "h2.SMO"
	h2.SMO.res$Temperature = Temperature.vec[j]
	h2.SMO.res$model.name = model.name
	model.covar.res = rbind(model.covar.res, h2.SMO.res)
	model.covar.res
	# c2 SMO PER TEMP
	c2.SMO = NULL
	c2.SMO.res = NULL
	c2.SMO = model$VCV[,SELECTION[1]] / (model$VCV[,SELECTION[1]] + model$VCV[,SELECTION[2]] + model$VCV[,SELECTION[3]] + 1)
	c2.SMO.res = round( data.frame(mean = mean(c2.SMO), HPDinterval(c2.SMO)), digits)
	c2.SMO.res$Parameter ="c2.SMO"
	c2.SMO.res$Temperature = Temperature.vec[j]
	c2.SMO.res$model.name = model.name
	model.covar.res = rbind(model.covar.res, c2.SMO.res)
	model.covar.res
	# RE SMO,LEN PER TEMP
	SELECTION = colnames(model$VCV[,
		which(
		grepl("units",colnames(model$VCV)))])
	SELECTION
	RE.SMO.LEN = NULL
	RE.SMO.LEN.res = NULL
	RE.SMO.LEN = model$VCV[,SELECTION[2]] / sqrt(model$VCV[,SELECTION[1]] *  model$VCV[,SELECTION[4]])
	RE.SMO.LEN.res = round( data.frame(mean = mean(RE.SMO.LEN), HPDinterval(RE.SMO.LEN)), digits)
	RE.SMO.LEN.res$Parameter ="RE.SMO.LEN"
	RE.SMO.LEN.res$Temperature = Temperature.vec[j]
	RE.SMO.LEN.res$model.name = model.name
	model.covar.res = rbind(model.covar.res, RE.SMO.LEN.res)
	model.covar.res
	# RC SMO,LEN PER TEMP
	SELECTION = colnames(model$VCV[,
		which(
		grepl("Tank",colnames(model$VCV)))])
	SELECTION
	RC.SMO.LEN = NULL
	RC.SMO.LEN.res = NULL
	RC.SMO.LEN = model$VCV[,SELECTION[2]] / sqrt(model$VCV[,SELECTION[1]] *  model$VCV[,SELECTION[4]])
	RC.SMO.LEN.res = round( data.frame(mean = mean(RC.SMO.LEN), HPDinterval(RC.SMO.LEN)), digits)
	RC.SMO.LEN.res$Parameter ="RC.SMO.LEN"
	RC.SMO.LEN.res$Temperature = Temperature.vec[j]
	RC.SMO.LEN.res$model.name = model.name
	model.covar.res = rbind(model.covar.res, RC.SMO.LEN.res)
	model.covar.res
	# RG SMO,LEN PER TEMP
	SELECTION = colnames(model$VCV[,
		which(
		grepl("DNA.ID.at.Tagging",colnames(model$VCV)))])
	SELECTION
	RG.SMO.LEN = NULL
	RG.SMO.LEN.res = NULL
	RG.SMO.LEN = model$VCV[,SELECTION[2]] / sqrt(model$VCV[,SELECTION[1]] *  model$VCV[,SELECTION[4]])
	RG.SMO.LEN.res = round( data.frame(mean = mean(RG.SMO.LEN), HPDinterval(RG.SMO.LEN)), digits)
	RG.SMO.LEN.res$Parameter ="RG.SMO.LEN"
	RG.SMO.LEN.res$Temperature = Temperature.vec[j]
	RG.SMO.LEN.res$model.name = model.name
	model.covar.res = rbind(model.covar.res, RG.SMO.LEN.res)
	model.covar.res
	RP.SMO.LEN = NULL
	RP.SMO.LEN.res = NULL
	RP.SMO.LEN = 
		RG.SMO.LEN * sqrt(h2.SMO * h2.LEN) + 
		RC.SMO.LEN * sqrt(c2.SMO * c2.LEN) + 
		RE.LEN.P2.SMO * sqrt( (1-(h2.SMO+c2.SMO))*(1-(h2.LEN+c2.LEN)) )
	RP.SMO.LEN.res = round( data.frame(mean = mean(RP.SMO.LEN), HPDinterval(RP.SMO.LEN)), digits)
	RP.SMO.LEN.res$Parameter ="RP.SMO.LEN"
	RP.SMO.LEN.res$Temperature = Temperature.vec[j]
	RP.SMO.LEN.res$model.name = model.name
	model.covar.res = rbind(model.covar.res, RP.SMO.LEN.res)
	model.covar.res
	} 
rownames(model.covar.res) = NULL
model.covar.res
}

# 3) BV + GxE model for MIG,LEN responses across temperatures
RUN.BV_model.covar.res = function(model.list= NULL, model.name.list= NULL, digits= 3) {
	require("coda")
	Temperature.vec = c("Cold","Warm")
	"BV_model.covar.res" = NULL
	"model.contrast.res" = NULL
	for(i in 1:length(model.name.list))
	{
	require("coda")
	model = model.list[[i]]
	model.name = model.name.list[[i]]
	# between-temperature parameters:
	# h2 SMO CONTRAST for males between temperatures
	SELECTION = colnames(model$VCV[,
		which(
		!grepl("traitLEN",colnames(model$VCV)))])
	SELECTION
	h2.TEMP.CONTRAST = NULL
	h2.TEMP.CONTRAST.res = NULL
	h2.TEMP.CONTRAST =  model$VCV[,SELECTION[3]] / (model$VCV[,SELECTION[3]] + model$VCV[,SELECTION[1]] + model$VCV[,SELECTION[7]] + 1) - model$VCV[,SELECTION[6]] / (model$VCV[,SELECTION[6]] + model$VCV[,SELECTION[2]] + model$VCV[,SELECTION[8]] + 1)
	h2.TEMP.CONTRAST.res = round( data.frame(mean = mean(h2.TEMP.CONTRAST), HPDinterval(h2.TEMP.CONTRAST)), digits)
	h2.TEMP.CONTRAST.res$Parameter = "h2.SMO - h2.SMO"
	h2.TEMP.CONTRAST.res$Temperature = "Cold-Warm"
	h2.TEMP.CONTRAST.res$model.name = model.name
	BV_model.covar.res = rbind(BV_model.covar.res, h2.TEMP.CONTRAST.res)
	BV_model.covar.res
	# h2 LEN.P2 CONTRAST for females between temperatures
	SELECTION = colnames(model$VCV[,
		which(
		!grepl("traitSMO",colnames(model$VCV)))] )
	SELECTION
	h2.TEMP.CONTRAST = NULL
	h2.TEMP.CONTRAST.res = NULL
	h2.TEMP.CONTRAST = model$VCV[,SELECTION[3]] / (model$VCV[,SELECTION[3]] + model$VCV[,SELECTION[1]] + model$VCV[,SELECTION[7]] ) - model$VCV[,SELECTION[6]] / (model$VCV[,SELECTION[6]] + model$VCV[,SELECTION[2]] + model$VCV[,SELECTION[8]] )
	h2.TEMP.CONTRAST.res = round(data.frame(mean = mean(h2.TEMP.CONTRAST), HPDinterval(h2.TEMP.CONTRAST)), digits)
	h2.TEMP.CONTRAST.res$Parameter = "h2.LEN - h2.LEN"
	h2.TEMP.CONTRAST.res$Temperature = "Cold-Warm"
	h2.TEMP.CONTRAST.res$model.name = model.name
	BV_model.covar.res = rbind(BV_model.covar.res, h2.TEMP.CONTRAST.res)
	BV_model.covar.res
	# RG between SMO Cold vs SMO in Warm
	SELECTION = colnames(model$VCV[,
		which(
		grepl("DNA.ID.at.Tagging",colnames(model$VCV)) & 
		!grepl("traitLEN",colnames(model$VCV))
		)])
	SELECTION
	RG.GxE.SMO = NULL
	RG.GxE.SMO.res = NULL
	RG.GxE.SMO = model$VCV[, SELECTION[2]]/sqrt(model$VCV[,SELECTION[1]]*model$VCV[,SELECTION[4]])
	RG.GxE.SMO.res = round(data.frame(mean = mean(RG.GxE.SMO), HPDinterval(RG.GxE.SMO)), digits)
	RG.GxE.SMO.res$Parameter ="RG.SMO.SMO"
	RG.GxE.SMO.res$Temperature = "Warm-Cold"
	RG.GxE.SMO.res$model.name = model.name
	BV_model.covar.res = rbind(BV_model.covar.res, RG.GxE.SMO.res)
	BV_model.covar.res
	# RG between LEN.P2 Cold vs LEN.P2 in Warm
	SELECTION = colnames(model$VCV[,
		which(
		grepl("DNA.ID.at.Tagging",colnames(model$VCV)) & 
		!grepl("traitSMO",colnames(model$VCV))
		)])
	SELECTION
	RG.GxE.LEN.P2 = NULL
	RG.GxE.LEN.P2.res = NULL
	RG.GxE.LEN.P2 = model$VCV[, SELECTION[2]]/sqrt(model$VCV[,SELECTION[1]]*model$VCV[,SELECTION[4]])
	RG.GxE.LEN.P2.res = round(data.frame(mean = mean(RG.GxE.LEN.P2), HPDinterval(RG.GxE.LEN.P2)), digits)
	RG.GxE.LEN.P2.res$Parameter ="RG.LEN.LEN"
	RG.GxE.LEN.P2.res$Temperature = "Warm-Cold"
	RG.GxE.LEN.P2.res$model.name = model.name
	BV_model.covar.res = rbind(BV_model.covar.res, RG.GxE.LEN.P2.res)
	BV_model.covar.res
	#### DELTA RG; DELTA RE; DELTA RP
	# DELTA RE for LEN.P2 vs SMO
	SELECTION = colnames(model$VCV[,
		which(
		 grepl("units",colnames(model$VCV)) )])
	SELECTION
	DELTA.RE.LEN.P2.SMO = NULL
	DELTA.RE.LEN.P2.SMO.res = NULL
	DELTA.RE.LEN.P2.SMO = (model$VCV[,SELECTION[2]] / sqrt(model$VCV[,SELECTION[1]] * model$VCV[,SELECTION[4]] )) - (model$VCV[,SELECTION[6]] / sqrt(model$VCV[,SELECTION[5]] * model$VCV[,SELECTION[8]] ))
	DELTA.RE.LEN.P2.SMO.res = round(data.frame(mean = mean(DELTA.RE.LEN.P2.SMO), HPDinterval(DELTA.RE.LEN.P2.SMO)), digits)
	DELTA.RE.LEN.P2.SMO.res$Parameter ="DELTA.RE.SMO.LEN_C_W"
	DELTA.RE.LEN.P2.SMO.res$Temperature = "Cold-Warm"
	DELTA.RE.LEN.P2.SMO.res$model.name = model.name
	BV_model.covar.res = rbind(BV_model.covar.res, DELTA.RE.LEN.P2.SMO.res)
	BV_model.covar.res
	# DELTA RG for SMO vs LEN.P2
	SELECTION = colnames(model$VCV[,
		which(
		 grepl("DNA.ID.at.Tagging",colnames(model$VCV)) )])
	SELECTION
	DELTA.RG.SMO.LEN = NULL
	DELTA.RG.SMO.LEN.res = NULL
	DELTA.RG.SMO.LEN = (model$VCV[,SELECTION[2]] / sqrt(model$VCV[,SELECTION[1]] * model$VCV[,SELECTION[6]] )) - (model$VCV[,SELECTION[12]] / sqrt(model$VCV[,SELECTION[11]] * model$VCV[,SELECTION[16]] ))
	DELTA.RG.SMO.LEN.res = round(data.frame(mean = mean(DELTA.RG.SMO.LEN), HPDinterval(DELTA.RG.SMO.LEN)), digits)
	DELTA.RG.SMO.LEN.res$Parameter ="DELTA.RG.SMO.LEN_C_W"
	DELTA.RG.SMO.LEN.res$Temperature = "Cold-Warm"
	DELTA.RG.SMO.LEN.res$model.name = model.name
	BV_model.covar.res = rbind(BV_model.covar.res, DELTA.RG.SMO.LEN.res)
	BV_model.covar.res
# temperature-specific parameters:	
	for(j in 1:length(Temperature.vec))
	{
	# RG for SMO vs LEN.P2
	SELECTION = colnames(model$VCV[,
		which(
		 grepl("DNA.ID.at.Tagging",colnames(model$VCV)) &
		!grepl(Temperature.vec[Temperature.vec!=Temperature.vec[j]],colnames(model$VCV)))])
	SELECTION
	RG.SMO.LEN = NULL
	RG.SMO.LEN.res = NULL
	RG.SMO.LEN = model$VCV[,SELECTION[2]] / sqrt(model$VCV[,SELECTION[1]] * model$VCV[,SELECTION[4]] )
	RG.SMO.LEN.res = round(data.frame(mean = mean(RG.SMO.LEN), HPDinterval(RG.SMO.LEN)), digits)
	RG.SMO.LEN.res$Parameter ="RG.SMO.LEN"
	RG.SMO.LEN.res$Temperature = Temperature.vec[j]
	RG.SMO.LEN.res$model.name = model.name
	BV_model.covar.res = rbind(BV_model.covar.res, RG.SMO.LEN.res)
	BV_model.covar.res
	# RE for LEN.P2 vs SMO
	SELECTION = colnames(model$VCV[,
		which(
		 grepl("units",colnames(model$VCV)) & 
		!grepl(Temperature.vec[Temperature.vec!=Temperature.vec[j]],colnames(model$VCV)))])
	SELECTION
	RE.LEN.P2.SMO = NULL
	RE.LEN.P2.SMO.res = NULL
	"RE.LEN.P2.SMO" = model$VCV[,SELECTION[2]] / sqrt(model$VCV[,SELECTION[1]] * model$VCV[,SELECTION[4]] )
	RE.LEN.P2.SMO.res = round(data.frame(mean = mean(RE.LEN.P2.SMO), HPDinterval(RE.LEN.P2.SMO)), digits)
	RE.LEN.P2.SMO.res$Parameter ="RE.SMO.LEN"
	RE.LEN.P2.SMO.res$Temperature = Temperature.vec[j]
	RE.LEN.P2.SMO.res$model.name = model.name
	BV_model.covar.res = rbind(BV_model.covar.res, RE.LEN.P2.SMO.res)
	BV_model.covar.res
	# h2 LEN.P2  PER TEMP
	SELECTION = colnames(model$VCV[,
		which(
		!grepl("traitSMO",colnames(model$VCV)) & 
		!grepl(Temperature.vec[Temperature.vec!=Temperature.vec[j]],colnames(model$VCV)))])
	SELECTION
	h2.LEN = NULL
	h2.LEN.res = NULL
	"h2.LEN" = model$VCV[,SELECTION[2]] / (model$VCV[,SELECTION[2]] + model$VCV[,SELECTION[1]] + model$VCV[,SELECTION[3]])
	h2.LEN.res = round(data.frame(mean = mean(h2.LEN), HPDinterval(h2.LEN)), digits)
	h2.LEN.res$Parameter ="h2.LEN"
	h2.LEN.res$Temperature = Temperature.vec[j]
	h2.LEN.res$model.name = model.name
	BV_model.covar.res = rbind(BV_model.covar.res, h2.LEN.res)
	BV_model.covar.res
	# h2 SMO PER TEMP
	SELECTION = colnames(model$VCV[,
		which(
		!grepl("traitLEN",colnames(model$VCV)) & 
		!grepl(Temperature.vec[Temperature.vec!=Temperature.vec[j]],colnames(model$VCV)))])
	SELECTION
	h2.SMO = NULL
	h2.SMO.res = NULL
	"h2.SMO" = model$VCV[,SELECTION[2]] / (model$VCV[,SELECTION[2]] + model$VCV[,SELECTION[1]] + model$VCV[,SELECTION[3]] + 1 )
	h2.SMO.res = round(data.frame(mean = mean(h2.SMO), HPDinterval(h2.SMO)), digits)
	h2.SMO.res$Parameter ="h2.SMO"
	h2.SMO.res$Temperature = Temperature.vec[j]
	h2.SMO.res$model.name = model.name
	BV_model.covar.res = rbind(BV_model.covar.res, h2.SMO.res)
	BV_model.covar.res
	# c2 LEN.P2 PER TEMP
	SELECTION = colnames(model$VCV[,
		which(
		!grepl("traitSMO",colnames(model$VCV)) &
		!grepl(Temperature.vec[Temperature.vec!=Temperature.vec[j]],colnames(model$VCV)))])
	SELECTION
	c2.LEN = NULL
	c2.LEN.res = NULL
	"c2.LEN" = model$VCV[,SELECTION[1]] / (model$VCV[,SELECTION[1]] + model$VCV[,SELECTION[2]] + model$VCV[,SELECTION[3]]  )
	c2.LEN.res = round(data.frame(mean = mean(c2.LEN), HPDinterval(c2.LEN)), digits)
	c2.LEN.res$Parameter ="c2.LEN"
	c2.LEN.res$Temperature = Temperature.vec[j]
	c2.LEN.res$model.name = model.name
	BV_model.covar.res = rbind(BV_model.covar.res, c2.LEN.res)
	BV_model.covar.res
	# c2 SMO PER TEMP
	SELECTION = colnames(model$VCV[,
		which(
		!grepl("traitLEN",colnames(model$VCV)) &
		!grepl(Temperature.vec[Temperature.vec!=Temperature.vec[j]],colnames(model$VCV)))])
	SELECTION
	c2.SMO = NULL
	c2.SMO.res = NULL
	"c2.SMO" = model$VCV[,SELECTION[1]] / (model$VCV[,SELECTION[1]] + model$VCV[,SELECTION[2]] + model$VCV[,SELECTION[3]] + 1 )
	c2.SMO.res = round(data.frame(mean = mean(c2.SMO), HPDinterval(c2.SMO)), digits)
	c2.SMO.res$Parameter ="c2.SMO"
	c2.SMO.res$Temperature = Temperature.vec[j]
	c2.SMO.res$model.name = model.name
	BV_model.covar.res = rbind(BV_model.covar.res, c2.SMO.res)
	BV_model.covar.res
	# RC between traits
	SELECTION = colnames(model$VCV[,
		which(
		!grepl(Temperature.vec[Temperature.vec!=Temperature.vec[j]],colnames(model$VCV)) & 
		grepl(".Tank",colnames(model$VCV) ))])
	SELECTION
	RC.SMO.LEN = NULL
	RC.SMO.LEN.res = NULL
	"RC.SMO.LEN" = model$VCV[,SELECTION[2]] / sqrt(model$VCV[,SELECTION[1]] * model$VCV[,SELECTION[4]] )
	RC.SMO.LEN.res = round(data.frame(mean = mean(RC.SMO.LEN), HPDinterval(RC.SMO.LEN)), digits)
	RC.SMO.LEN.res$Parameter ="RC.SMO.LEN"
	RC.SMO.LEN.res$Temperature = Temperature.vec[j]
	RC.SMO.LEN.res$model.name = model.name
	BV_model.covar.res = rbind(BV_model.covar.res, RC.SMO.LEN.res)
	BV_model.covar.res
	# RP LEN.P2 - SMO 
	RP.SMO.LEN.P2 = NULL
	RP.SMO.LEN.P2.res = NULL
	"RP.SMO.LEN.P2" = 
		RG.SMO.LEN * sqrt(h2.SMO * h2.LEN) + 
		RC.SMO.LEN * sqrt(c2.SMO * c2.LEN) + 
		RE.LEN.P2.SMO * sqrt( (1-(h2.SMO+c2.SMO))*(1-(h2.LEN+c2.LEN)) )
	RP.SMO.LEN.P2.res = round(data.frame(mean = mean(RP.SMO.LEN.P2), HPDinterval(RP.SMO.LEN.P2)), digits)
	RP.SMO.LEN.P2.res$Parameter ="RP.SMO.LEN"
	RP.SMO.LEN.P2.res$Temperature = Temperature.vec[j]
	RP.SMO.LEN.P2.res$model.name = model.name
	BV_model.covar.res = rbind(BV_model.covar.res, RP.SMO.LEN.P2.res)
	BV_model.covar.res
	if(Temperature.vec[1] != "Cold") stop("Temperature.vec has wrong order for temperatures, cold needs to be first")
	if(Temperature.vec[j] == "Cold") # first round: assign cold estimates, keep them
	{
	RP.SMO.LEN.P2.cold	= NULL
	RP.SMO.LEN.P2.cold	= RP.SMO.LEN.P2
	} else { # second round: make cold-warm contrast
	# DELTA RP LEN.P2 - SMO 
	DELTA.RP.SMO.LEN.P2 = NULL
	DELTA.RP.SMO.LEN.P2.res = NULL
	DELTA.RP.SMO.LEN.P2 = RP.SMO.LEN.P2.cold - RP.SMO.LEN.P2
	DELTA.RP.SMO.LEN.P2.res = round(data.frame(mean = mean(DELTA.RP.SMO.LEN.P2), HPDinterval(DELTA.RP.SMO.LEN.P2)), digits)
	DELTA.RP.SMO.LEN.P2.res$Parameter ="DELTA.RP.SMO.LEN_C_W"
	DELTA.RP.SMO.LEN.P2.res$Temperature = "Cold-Warm"
	DELTA.RP.SMO.LEN.P2.res$model.name = model.name
	BV_model.covar.res = rbind(BV_model.covar.res, DELTA.RP.SMO.LEN.P2.res)
	BV_model.covar.res 
	} } } 
rownames(BV_model.covar.res) = NULL
BV_model.covar.res
}

############################################################################################
####
#### define data path; load data
####
############################################################################################

# define path
DataPath = NULL
DataPath = "" # define path to "MigrantData.txt" "MigrantDataPedigree.txt"

# load data & pedigree
# beware: MIG in manuscript = SMO in data (MIG - Migrant <-> SMO - Smolt)
"MigrantData" = NULL
MigrantData = read.table(paste0(DataPath,"MigrantData.txt"), header= TRUE, sep="\t")
"MigrantDataPedigree" = NULL
MigrantDataPedigree = read.table(paste0(DataPath,"MigrantDataPedigree.txt"), header= TRUE, sep="\t")
dim(MigrantData)
# [1] 663  13

# Make A^-1 from pedigree
library("MCMCglmm")
"MigrantDataPedigree.MCMCglmm.Ainv" = NULL
MigrantDataPedigree.MCMCglmm.Ainv = inverseA(MigrantDataPedigree[,1:3])$Ainv 

############################################################################################
####
#### Figure S2 - UV model, no GxE for MIG response per temperature
####
############################################################################################

# subset data & pedigree Cold
"MigrantData_Cold" = NULL
MigrantData_Cold = droplevels(MigrantData[which(MigrantData$Temperature == "Cold"),])
MigrantData_Cold$DNA.ID.at.Tagging= as.character(MigrantData_Cold$DNA.ID.at.Tagging)
MigrantDataPedigree$DNA.ID.at.Tagging= as.character(MigrantDataPedigree$DNA.ID.at.Tagging)
"MigrantDataPedigree_Cold" = NULL
MigrantDataPedigree_Cold = MigrantDataPedigree
MigrantDataPedigree_Cold$FLAG = "REMOVE"
MigrantDataPedigree_Cold[which( MigrantDataPedigree_Cold$DNA.ID.at.Tagging %in% MigrantData_Cold$DNA.ID.at.Tagging ),]$FLAG = "KEEP"
MigrantDataPedigree_Cold[grep("OUL.", MigrantDataPedigree_Cold$DNA.ID.at.Tagging, fixed=TRUE),]$FLAG = "KEEP"
MigrantDataPedigree_Cold[grep("ML_TAI_17_", MigrantDataPedigree_Cold$DNA.ID.at.Tagging, fixed=TRUE),]$FLAG = "KEEP"
MigrantDataPedigree_Cold = droplevels(MigrantDataPedigree_Cold[which( MigrantDataPedigree_Cold$FLAG != "REMOVE" ),])
MigrantDataPedigree_Cold$FLAG = NULL
# Make A^-1 from pedigree
"MigrantDataPedigree_Cold.MCMCglmm.Ainv" = NULL
MigrantDataPedigree_Cold.MCMCglmm.Ainv.Ainv = inverseA(MigrantDataPedigree_Cold[,1:3])$Ainv 

# subset data & pedigree Warm
"MigrantData_Warm" = NULL
MigrantData_Warm = droplevels(MigrantData[which(MigrantData$Temperature == "Warm"),])
MigrantData_Warm$DNA.ID.at.Tagging= as.character(MigrantData_Warm$DNA.ID.at.Tagging)
MigrantDataPedigree$DNA.ID.at.Tagging= as.character(MigrantDataPedigree$DNA.ID.at.Tagging)
"MigrantDataPedigree_Warm" = NULL
MigrantDataPedigree_Warm = MigrantDataPedigree
MigrantDataPedigree_Warm$FLAG = "REMOVE"
MigrantDataPedigree_Warm[which( MigrantDataPedigree_Warm$DNA.ID.at.Tagging %in% MigrantData_Warm$DNA.ID.at.Tagging ),]$FLAG = "KEEP"
MigrantDataPedigree_Warm[grep("OUL.", MigrantDataPedigree_Warm$DNA.ID.at.Tagging, fixed=TRUE),]$FLAG = "KEEP"
MigrantDataPedigree_Warm[grep("ML_TAI_17_", MigrantDataPedigree_Warm$DNA.ID.at.Tagging, fixed=TRUE),]$FLAG = "KEEP"
MigrantDataPedigree_Warm = droplevels(MigrantDataPedigree_Warm[which( MigrantDataPedigree_Warm$FLAG != "REMOVE" ),])
MigrantDataPedigree_Warm$FLAG = NULL
# Make A^-1 from pedigree
"MigrantDataPedigree_Warm.MCMCglmm.Ainv.Ainv" = NULL
MigrantDataPedigree_Warm.MCMCglmm.Ainv = inverseA(MigrantDataPedigree_Warm[,1:3])$Ainv 

# UV prior, no GxE [for both Cold and Warm subset models]
prior.UNI.X21 = NULL
prior.UNI.X21 = list(
	R = list(V = 1, fix= 1), 
	G = list(G1 = list(V= 1, nu= 1000, alpha.mu= 0, alpha.V= 1),
			 G2 = list(V= 1, nu= 1000, alpha.mu= 0, alpha.V= 1))) 
prior.UNI.X21

# settings
SAMPLES = 5000+10000
BURNIN = 0
THIN = 100 
NITT = BURNIN + THIN*SAMPLES
NITT

# model [example for Warm]; use multi-chain script as for BIV + GxE model below
# name: SmoltBin.UNI_Warm
# name: SmoltBin.UNI_Cold
	MCMCglmm(
		fixed= SMO ~ 1,
		random= ~ Tank  
				+ DNA.ID.at.Tagging,	
		rcov=   ~ units,
		data= MigrantData_Warm, # Warm version
		# data= MigrantData_Cold, # Cold version
		ginverse= list(DNA.ID.at.Tagging = MigrantDataPedigree_Warm.MCMCglmm.Ainv), # Warm version
		# ginverse= list( DNA.ID.at.Tagging = MigrantDataPedigree_Cold.MCMCglmm.Ainv ), # Cold version
		trunc= TRUE,
		# pr= TRUE,
		family= "threshold",
		prior= prior.UNI.X21,
			nitt= NITT,
			burnin= BURNIN,
			thin= THIN)

# extract covar parameters (c2, h2) after post-processing chains: 
# Warm model
model = SmoltBin.UNI_Warm
temp.UNI.Warm = NULL
temp.UNI.Warm = data.frame(
	mean = 
	round(rbind(mean(model$VCV[,"DNA.ID.at.Tagging"] / (rowSums(model$VCV) + 1) ),
				mean(model$VCV[,"Tank"] / (rowSums(model$VCV) + 1) )),3),
  	round(rbind(HPDinterval(model$VCV[,"DNA.ID.at.Tagging"] / (rowSums(model$VCV) + 1) ),
				HPDinterval(model$VCV[,"Tank"] / (rowSums(model$VCV) + 1) )),3),
	Parameter = c("h2.SMO","c2.SMO"),
	Temperature = c("Warm","Warm"),
	model.name = model.name.list[[1]])
temp.UNI.Warm = temp.UNI.Warm[order(temp.UNI.Warm$model.name),]
temp.UNI.Warm = temp.UNI.Warm[order(temp.UNI.Warm$Temperature),]
temp.UNI.Warm = temp.UNI.Warm[order(temp.UNI.Warm$Parameter),]
temp.UNI.Warm

# Cold model
model = SmoltBin.UNI_Cold
temp.UNI.Cold = NULL
temp.UNI.Cold = data.frame(
	mean = 
	round(rbind(mean(model$VCV[,"DNA.ID.at.Tagging"] / (rowSums(model$VCV) + 1) ),
				mean(model$VCV[,"Tank"] / (rowSums(model$VCV) + 1) )),3),
  	round(rbind(HPDinterval(model$VCV[,"DNA.ID.at.Tagging"] / (rowSums(model$VCV) + 1) ),
				HPDinterval(model$VCV[,"Tank"] / (rowSums(model$VCV) + 1) )),3),
	Parameter = c("h2.SMO","c2.SMO"),
	Temperature = c("Cold","Cold"),
	model.name = model.name.list[[1]])
temp.UNI.Cold = temp.UNI.Cold[order(temp.UNI.Cold$model.name),]
temp.UNI.Cold = temp.UNI.Cold[order(temp.UNI.Cold$Temperature),]
temp.UNI.Cold = temp.UNI.Cold[order(temp.UNI.Cold$Parameter),]
temp.UNI.Cold		

########################################################################
####
#### Table 1, Figure 3, Table S1, Figure S2 - UV + GxE model for MIG response [incl model selection]
####
########################################################################

prior.UV = NULL
prior.UV = list(
	R = list(V = diag(2), fix= 1), 
	G = list(G1 = list(V= diag(2), nu= 3, alpha.mu= c(0,0), alpha.V= diag(2)),
			 G2 = list(V= diag(2), nu= 3, alpha.mu= c(0,0), alpha.V= diag(2)),
			 G3 = list(V= diag(2), nu= 3, alpha.mu= c(0,0), alpha.V= diag(2)))) 
prior.UV

prior.UV.reduced = NULL
prior.UV.reduced = list(
	R = list(V = diag(2), fix= 1), 
	G = list(G1 = list(V= diag(2), nu= 3, alpha.mu= c(0,0), alpha.V= diag(2)),
			 G3 = list(V= diag(2), nu= 3, alpha.mu= c(0,0), alpha.V= diag(2)))) 
prior.UV.reduced

# settings
SAMPLES = 10*1000+10000
BURNIN = 0
THIN = 100 
NITT = BURNIN + THIN*SAMPLES
NITT

# Table 1 - FULL model; use parallel script as for BIV + GxE model below
	MCMCglmm(
		fixed= SMO ~ 1 + Temperature * FeedRestriction * MAT,
		random= ~ idh(Temperature):Tank
				+ us(Temperature):Dam
				+ us(Temperature):DNA.ID.at.Tagging, 
		rcov=~ idh(Temperature):units,
		data= MigrantData,
 		trunc= TRUE,
		# pr= TRUE,
		ginverse= list(DNA.ID.at.Tagging = MigrantDataPedigree.MCMCglmm.Ainv),
		family= "threshold", 
		prior= prior.UV,
			nitt= NITT,
			burnin= BURNIN,
			thin= THIN)
		
# Figure 3 - REDUCED model; use parallel script as for BIV + GxE model below
	MCMCglmm(
		fixed= SMO ~ 1 + Temperature,
		random= ~ idh(Temperature):Tank
				+ us(Temperature):DNA.ID.at.Tagging, 
		rcov=~ idh(Temperature):units,
		data= MigrantData,
 		trunc= TRUE,
		# pr= TRUE,
		ginverse= list(DNA.ID.at.Tagging = MigrantDataPedigree.MCMCglmm.Ainv),
		family= "threshold", 
		prior= prior.UV.reduced,
			nitt= NITT,
			burnin= BURNIN,
			thin= THIN)
		
# Figure 2 (& Figure 3) - REDUCED model including length as a covariate; use parallel script as for BIV + GxE model below
	MCMCglmm(
		fixed= SMO ~ 1 + Temperature*LEN.P2,
		random= ~ idh(Temperature):Tank
				+ us(Temperature):DNA.ID.at.Tagging, 
		rcov=~ idh(Temperature):units,
		data= MigrantData,
 		trunc= TRUE,
		# pr= TRUE,
		ginverse= list(DNA.ID.at.Tagging = MigrantDataPedigree.MCMCglmm.Ainv),
		family= "threshold", 
		prior= prior.UV.reduced,
			nitt= NITT,
			burnin= BURNIN,
			thin= THIN)
		
# Table S1 - model 1: REDUCED model testing vgll3 effects; use parallel script as for BIV + GxE model below
	MCMCglmm(
		fixed= SMO ~ 1 + Temperature + VGLL3TOP,
		random= ~ idh(Temperature):Tank
				+ us(Temperature):DNA.ID.at.Tagging, 
		rcov=~ idh(Temperature):units,
		data= MigrantData,
 		trunc= TRUE,
		# pr= TRUE,
		ginverse= list(DNA.ID.at.Tagging = MigrantDataPedigree.MCMCglmm.Ainv),
		family= "threshold", 
		prior= prior.UV.reduced,
			nitt= NITT,
			burnin= BURNIN,
			thin= THIN)
		
# Table S1 - model 2: REDUCED model including length as a covariate and testing vgll3 effects; use parallel script as for BIV + GxE model below
	MCMCglmm(
		fixed= SMO ~ 1 + Temperature*LEN.P2 + VGLL3TOP,
		random= ~ idh(Temperature):Tank
				+ us(Temperature):DNA.ID.at.Tagging, 
		rcov=~ idh(Temperature):units,
		data= MigrantData,
 		trunc= TRUE,
		# pr= TRUE,
		ginverse= list(DNA.ID.at.Tagging = MigrantDataPedigree.MCMCglmm.Ainv),
		family= "threshold", 
		prior= prior.UV.reduced,
			nitt= NITT,
			burnin= BURNIN,
			thin= THIN)		

# extract covar parameters: use "RUN.model.covar.res_SMO"
		
########################################################################
####
#### Figure S2 - BV model, no GxE for MIG,LEN responses per temperature
####
########################################################################

# BV prior, no GxE
prior.A_NoGxE = NULL
prior.A_NoGxE = list(
	R = list(R1 = list(V= diag(2), nu= 2, fix= 2)), 
	G = list(G1 = list(V= diag(2), nu= 3, alpha.mu= c(0,0), alpha.V= diag(c(10,1))),
 			 G2 = list(V= diag(2), nu= 3, alpha.mu= c(0,0), alpha.V= diag(c(10,1)))))
prior.A_NoGxE

# settings
SAMPLES = 10*1000+10000
BURNIN = 0
THIN = 100 
NITT = BURNIN + THIN*SAMPLES
NITT

# model [example for Warm]; use multi-chain script as for BIV + GxE model below
	MCMCglmm(
		fixed= cbind( LEN.P2 , SMO ) ~ trait-1, 
		random= ~ us(trait):Tank  
				+ us(trait):DNA.ID.at.Tagging,	
		rcov= ~ us(trait):units,
		data= MigrantData_Warm, 
		# data= MigrantData_Cold, 
		ginverse= list(DNA.ID.at.Tagging = MigrantDataPedigree_Warm.MCMCglmm.Ainv),
		# ginverse= list(DNA.ID.at.Tagging = MigrantDataPedigree_Cold.MCMCglmm.Ainv),
		trunc= TRUE,
		# pr= TRUE,
		family= c("gaussian" , "threshold"),
		prior= prior.A_NoGxE ,
			nitt= NITT,
			burnin= BURNIN,
			thin= THIN)
		
# extract covar parameters: use "RUN.model.covar.res_BIV.NoGxE"

############################################################################################
####
#### Figure 3 - BV + GxE model for MIG,LEN responses across temperatures 
#### Figure S3, Figure S4 - Sensitivity analysis
####
############################################################################################

# BV priors, GxE

## used BV prior (Figure 3)
prior.A = NULL
prior.A = list(
	R = list(R1 = list(V= diag(2), nu= 2, fix= 2),
			 R2 = list(V= diag(2), nu= 2, fix= 2)), 
	G = list(G1 = list(V= diag(2), nu= 3, alpha.mu= rep(0,2), alpha.V= diag(c(10,1))),
			 G2 = list(V= diag(2), nu= 3, alpha.mu= rep(0,2), alpha.V= diag(c(10,1))),
 			 G3 = list(V= diag(4), nu= 5, alpha.mu= rep(0,4), alpha.V= diag(rep(c(10,1),2)))))
prior.A

## alternative priors
# varying nu (Figure S4):
prior.B = NULL
prior.B = list(
	R = list(R1 = list(V= diag(2), nu= 2, fix= 2),
			 R2 = list(V= diag(2), nu= 2, fix= 2)), 
	G = list(G1 = list(V= diag(2), nu= 2, alpha.mu= rep(0,2), alpha.V= diag(c(10,1))),
			 G2 = list(V= diag(2), nu= 2, alpha.mu= rep(0,2), alpha.V= diag(c(10,1))),
 			 G3 = list(V= diag(4), nu= 4, alpha.mu= rep(0,4), alpha.V= diag(rep(c(10,1),2)))))
prior.B

prior.C = NULL
prior.C = list(
	R = list(R1 = list(V= diag(2), nu= 3, fix= 2),
			 R2 = list(V= diag(2), nu= 3, fix= 2)), 
	G = list(G1 = list(V= diag(2), nu= 3, alpha.mu= rep(0,2), alpha.V= diag(c(10,1))),
			 G2 = list(V= diag(2), nu= 3, alpha.mu= rep(0,2), alpha.V= diag(c(10,1))),
 			 G3 = list(V= diag(4), nu= 5, alpha.mu= rep(0,4), alpha.V= diag(rep(c(10,1),2)))))
prior.C

prior.D = NULL
prior.D = list(
	R = list(R1 = list(V= diag(2), nu= 3, fix= 2),
			 R2 = list(V= diag(2), nu= 3, fix= 2)), 
	G = list(G1 = list(V= diag(2), nu= 2, alpha.mu= rep(0,2), alpha.V= diag(c(10,1))),
			 G2 = list(V= diag(2), nu= 2, alpha.mu= rep(0,2), alpha.V= diag(c(10,1))),
 			 G3 = list(V= diag(4), nu= 4, alpha.mu= rep(0,4), alpha.V= diag(rep(c(10,1),2)))))
prior.D

# varying alpha.V (Figure S3):
prior.E = NULL
prior.E = list(
	R = list(R1 = list(V= diag(2), nu= 2, fix= 2),
			 R2 = list(V= diag(2), nu= 2, fix= 2)), 
	G = list(G1 = list(V= diag(2), nu= 3, alpha.mu= rep(0,2), alpha.V= diag(c(10, 10))),
			 G2 = list(V= diag(2), nu= 3, alpha.mu= rep(0,2), alpha.V= diag(c(10, 10))),
 			 G3 = list(V= diag(4), nu= 5, alpha.mu= rep(0,4), alpha.V= diag(rep(c(10, 10),2)))))
prior.E

prior.F = NULL
prior.F = list(
	R = list(R1 = list(V= diag(2), nu= 2, fix= 2),
			 R2 = list(V= diag(2), nu= 2, fix= 2)), 
	G = list(G1 = list(V= diag(2), nu= 3, alpha.mu= rep(0,2), alpha.V= diag(c(10, 0.1))),
			 G2 = list(V= diag(2), nu= 3, alpha.mu= rep(0,2), alpha.V= diag(c(10, 0.1))),
 			 G3 = list(V= diag(4), nu= 5, alpha.mu= rep(0,4), alpha.V= diag(rep(c(10, 0.1),2)))))
prior.F

prior.G = NULL
prior.G = list(
	R = list(R1 = list(V= diag(2), nu= 2, fix= 2),
			 R2 = list(V= diag(2), nu= 2, fix= 2)), 
	G = list(G1 = list(V= diag(2), nu= 3, alpha.mu= rep(0,2), alpha.V= diag(c(10, 0.01))),
			 G2 = list(V= diag(2), nu= 3, alpha.mu= rep(0,2), alpha.V= diag(c(10, 0.01))),
 			 G3 = list(V= diag(4), nu= 5, alpha.mu= rep(0,4), alpha.V= diag(rep(c(10, 0.01),2)))))
prior.G

prior.H = NULL
prior.H = list(
	R = list(R1 = list(V= diag(2), nu= 2, fix= 2),
			 R2 = list(V= diag(2), nu= 2, fix= 2)), 
	G = list(G1 = list(V= diag(2), nu= 3, alpha.mu= rep(0,2), alpha.V= diag(c(10, 100))),
			 G2 = list(V= diag(2), nu= 3, alpha.mu= rep(0,2), alpha.V= diag(c(10, 100))),
 			 G3 = list(V= diag(4), nu= 5, alpha.mu= rep(0,4), alpha.V= diag(rep(c(10, 100),2)))))
prior.H

# settings BV + GxE models
SAMPLES = 10*5000+10000
BURNIN = 0
THIN = 100 
NITT = BURNIN + THIN*SAMPLES
NITT

#### multi-chain script [used for all MCMC models]
# four chains per model, store in list; change exported objects (e.g., prior, data) and model acccordingly

cl = makeCluster(4) # 4 cpu for 4 chains

clusterEvalQ(cl, library("MCMCglmm")) 
clusterExport(cl, "MigrantData")
clusterExport(cl, "MigrantDataPedigree.MCMCglmm.Ainv")
clusterExport(cl, "prior.H")
clusterExport(cl, "SAMPLES")
clusterExport(cl, "BURNIN")
clusterExport(cl, "THIN")
clusterExport(cl, "NITT")

BV.model.GxE_prior.A_LIST4 = NULL
BV.model.GxE_prior.A_LIST4 = parLapply(cl= cl, 1:4, function(i) {
		
	MCMCglmm(
		fixed= cbind( LEN.P2 , SMO ) ~  trait-1
			+ trait:Temperature,
		random= ~ us(trait:at.level(Temperature, "Cold")):Tank 
				+ us(trait:at.level(Temperature, "Warm")):Tank  
				+ us(trait:Temperature):DNA.ID.at.Tagging,	
		rcov=   ~ us(trait:at.level(Temperature, "Cold")):units
				+ us(trait:at.level(Temperature, "Warm")):units,
		data= MigrantData, 
		ginverse= list( DNA.ID.at.Tagging = MigrantDataPedigree.MCMCglmm.Ainv),
		trunc= TRUE,
		# pr= TRUE,
		family= c("gaussian" , "threshold"),
		prior= prior.H ,
			nitt= NITT,
			burnin= BURNIN,
			thin= THIN)
			
})
stopCluster(cl)

# post-processing of model list
model.list = NULL
model.list = SmoltBin.BIV_OUL.fixed.red2_prior.A_LIST4

# subset for Sol posteriors
Sol.list = lapply(model.list, function(x) x$Sol)
Sol.list = do.call(coda:::mcmc.list, Sol.list)
str(Sol.list)

# assess mixing & posterior distributions (Sol), chains overlaid
plot(Sol.list)

rm(Sol.list)

# subset for VCV posteriors
VCV.list = lapply(model.list, function(x) x$VCV[,-which(grepl("units",colnames(x$VCV)) & grepl("SMO",colnames(x$VCV)))])
VCV.list = do.call(coda:::mcmc.list, VCV.list)
str(VCV.list)

# assess mixing & posterior distributions (VCV), chains overlaid
plot(VCV.list)

# Gelman and Rubin's convergence diagnostic -> convergence reached if upper limit is close to 1
gelman.diag.res = NULL
for(i in 1:ncol(VCV.list[[1]]))
{
gelman.diag.res.temp = data.frame(Term = attr(VCV.list[[1]],"dimnames")[[2]][i])
gelman.diag.res.temp = cbind(gelman.diag.res.temp, gelman.diag(VCV.list[,i])$psrf ) 
gelman.diag.res = rbind( gelman.diag.res , gelman.diag.res.temp )
}
gelman.diag.res
range(gelman.diag.res$"Point est.")
range(gelman.diag.res$"Upper C.I.")

# Gelman-Rubin-Brooks plot -> determine required burnin for consistent scale reduction factor < 1.1 across chains
par(ask=TRUE)
gelman.plot(VCV.list, auto.layout=TRUE)

rm(VCV.list)

# burnin and thin (adjust accordingly)
model.list = lapply(model.list, postburnin.MCMCglmm, burnin= 500000)
model.list = lapply(model.list, postthin.MCMCglmm, thin= 1000)

# assess autocorr
cor.temp = NULL
for(i in 1:4){
cor.temp = rbind(cor.temp,round(autocorr.diag(model.list[[i]]$VCV)[2,],3))
}
cor.temp = data.frame(t(cor.temp))
cor.temp$max.cor = cbind(apply(cor.temp, 1, max))
cor.temp
range(cor.temp$max,na.rm=TRUE)

# combine chains
"BV.model.GxE_prior.A" = NULL
BV.model.GxE_prior.A = model.list[[1]]

THIN = thin(BV.model.GxE_prior.A$VCV) 
THIN
START = start(BV.model.GxE_prior.A$VCV)
START
END = nrow(BV.model.GxE_prior.A$VCV) * THIN + START-1
END
END.full = nrow(BV.model.GxE_prior.A$VCV) *4 * THIN + START-1
END.full

BV.model.GxE_prior.A$VCV = mcmc(rbind(BV.model.GxE_prior.A$VCV, model.list[[2]]$VCV, model.list[[3]]$VCV, model.list[[4]]$VCV), start=START, end=END.full, thin=THIN)
BV.model.GxE_prior.A$Sol = mcmc(rbind(BV.model.GxE_prior.A$Sol, model.list[[2]]$Sol, model.list[[3]]$Sol, model.list[[4]]$Sol), start=START, end=END.full, thin=THIN)
summary(BV.model.GxE_prior.A$VCV[,1])

# assess combined chains
# autocor
cor.temp = NULL
cor.temp = round(autocorr.diag(BV.model.GxE_prior.A$VCV)[2,],3)
range(cor.temp,na.rm=TRUE)

# posterior distributions
plot(BV.model.GxE_prior.A$Sol[,1:nrow(summary(BV.model.GxE_prior.A)$solution)])
plot(BV.model.GxE_prior.A$VCV)

# summary
summary(BV.model.GxE_prior.A)

# extract covar parameters: use "RUN.BV_model.covar.res"
# example, can do for all BIV GxE models at once for sensitivity analysis by including all models in lists

BV.model.GxE_prior.A_COVARS = RUN.BV_model.covar.res(
	model.list= list(BV.model.GxE_prior.A), 
	model.name.list= list("BV.model.GxE_prior.A"), 
	digits= 3)
BV.model.GxE_prior.A_COVARS


########################################################################
####
#### predict parameters on observed scale for BV + GxE model for MIG,LEN responses across temperatures
####
########################################################################

library("QGglmm")
library("MCMCglmm")

library("foreach")
library("doParallel")

"model" = NULL
model = BV.model.GxE_prior.A

# N iterations
N = nrow(model$Sol)
N

# create list
Predict.N = NULL
Predict.N[[1]] = vector("list", N)
Predict.N[[2]] = vector("list", N)
Predict.N[[3]] = vector("list", N)
Predict.N[[4]] = vector("list", N)
Predict.N

# predict from model (marginal)
# second trait is the binary
bin.trait.seq = seq(nrow(MigrantData)+1, nrow(MigrantData)*2, by=1)
bin.trait.seq

# make list with data per iteration
for(i in 1:N) {
	# iteration
	Predict.N[[1]][[i]] = i 
	# predict per iteration
	Predict.N[[2]][[i]] = predict.MCMCglmm(model,
		it = i,
		type = "terms", 
		marginal = model$Random$formula)[bin.trait.seq]
	# G per iteration
	SELECTION = colnames(model$VCV[,
		which( 
		!grepl("traitLEN",colnames(model$VCV)) & 
		 grepl("DNA.ID.at.Tagging",colnames(model$VCV)) )]) 
	vcv.G = matrix(c(model$VCV[i , SELECTION[1]] ,
 					model$VCV[i , SELECTION[2]] ,
 					model$VCV[i , SELECTION[3]] ,
					model$VCV[i , SELECTION[4]] ), 2, 2)
	vcv.G
	Predict.N[[3]][[i]] = vcv.G
	# other components of VP
	# T per iteration
	SELECTION = colnames(model$VCV[,
		which(
		!grepl("traitLEN",colnames(model$VCV)) & 
		 grepl("Tank",colnames(model$VCV)) )])
	vcv.T = matrix(c(model$VCV[i , SELECTION[1]], 0,
 				 0, model$VCV[i , SELECTION[2]] ), 2, 2)	
	vcv.T
	# R per iteration
	SELECTION = colnames(model$VCV[,
		which(
		!grepl("traitLEN",colnames(model$VCV)) & 
		 grepl("units",colnames(model$VCV)) )])
	vcv.R = matrix(c(model$VCV[i , SELECTION[1]], 0,
 				 0, model$VCV[i , SELECTION[2]] ), 2, 2)	
	vcv.R	
	# P per iteration
	vcv.P = vcv.G + vcv.T + vcv.R
	Predict.N[[4]][[i]] = vcv.P
	}
Predict.N[[1]][N]
Predict.N[[2]][N]
Predict.N[[3]][N]
Predict.N[[4]][N]

# run "QGmvparams" for each of the N iterations (use as many CPU as possible, takes long)
library("foreach")
N.cpu = 5
cl = makeCluster( N.cpu )
registerDoParallel(cl)

Predict.N.out = NULL
Predict.N.out = 
foreach(i = 1:N, .packages= c("MCMCglmm","QGglmm","foreach")) %dopar% {
	# i = 1
	predict = matrix(       cbind(ifelse(MigrantData$Temperature == "Cold", Predict.N[[2]][[i]], -1) , ifelse(MigrantData$Temperature == "Warm", Predict.N[[2]][[i]], -1)), ncol=2)
	mask =    matrix( is.na(cbind(ifelse(MigrantData$Temperature == "Cold", Predict.N[[2]][[i]], NA) , ifelse(MigrantData$Temperature == "Warm", Predict.N[[2]][[i]], NA))),ncol=2)
	G = Predict.N[[3]][[i]]
	P = Predict.N[[4]][[i]]
	temp = NULL
	temp = QGmvparams(predict = predict, mask = mask, vcv.G = G, vcv.P = P,
			model = c("binom1.probit", "binom1.probit"), verbose = FALSE)
	temp$"h2.obs" = temp$vcv.G.obs / temp$vcv.P.obs
	temp$"CORgen.obs" = temp$vcv.G.obs[1,2] / sqrt(temp$vcv.G.obs[1,1]*temp$vcv.G.obs[2,2])
	temp$i = i
	temp
	}
stopCluster(cl)

names(Predict.N.out)[1] = "Temperature"
Predict.N.out$Temperature = ifelse(Predict.N.out$Temperature == 1, "Cold",
								ifelse(Predict.N.out$Temperature == 2, "Warm", NA))
names(Predict.N.out)[2]  = "mean.obs"
names(Predict.N.out)[3]  = "vcv.P.obs.Cold"
names(Predict.N.out)[4]  = "vcv.P.obs.Warm"
names(Predict.N.out)[5]  = "vcv.G.obs.Cold"
names(Predict.N.out)[6]  = "vcv.G.obs.Warm"
names(Predict.N.out)[7]  = "vcv.h2.obs.Cold"
names(Predict.N.out)[8]  = "vcv.h2.obs.Warm"
names(Predict.N.out)[9]  = "CORgen.obs"
names(Predict.N.out)[10] = "Iteration"

# get difference in probs between environments
temp = NULL
temp = unstack(form = mean.obs ~ Temperature, Predict.N.out)
temp$DeltaT.SMO = temp$Warm / temp$Cold
data.frame(mean = round(mean(temp$DeltaT.SMO),2), round(HPDinterval(as.mcmc(temp$DeltaT.SMO)),2))

# means are in different rows, correlation is repeated, all others are in 2x2 matrix
"Predict.N.out_Cold" = NULL
Predict.N.out_Cold = Predict.N.out[which(Predict.N.out$Temperature == "Cold"),]
head(Predict.N.out_Cold)

# means are in different rows, correlation is repeated, all others are in 2x2 matrix
"Predict.N.out_Warm" = NULL
Predict.N.out_Warm = Predict.N.out[which(Predict.N.out$Temperature == "Warm"),]
head(Predict.N.out_Warm)

# extract covar parameters
dim(Predict.N.out)
"DIM.1" = NULL
DIM.1 = max(Predict.N.out$Iteration)
Predict.N.out = unique(Predict.N.out)
dim(Predict.N.out_Cold)
dim(Predict.N.out_Warm)

digits = 3
mean.obs.95perc_Cold = NULL
mean.obs.95perc_Cold = cbind( "mean" = round( colMeans(as.mcmc( Predict.N.out_Cold[,c(2,grep("Cold",colnames(Predict.N.out_Cold))),], start = 1, end = DIM.1, thin = 1 ) ) ,digits) , round( HPDinterval(as.mcmc( Predict.N.out_Cold[,c(2,grep("Cold",colnames(Predict.N.out_Cold))),], start = 1, end = DIM.1, thin = 1  ) ) ,digits) )

mean.obs.95perc_Warm = NULL
mean.obs.95perc_Warm = cbind( "mean" = round( colMeans(as.mcmc( Predict.N.out_Warm[,c(2,grep("Warm",colnames(Predict.N.out_Warm))),], start = 1, end = DIM.1, thin = 1 ) ) ,digits) , round( HPDinterval(as.mcmc( Predict.N.out_Warm[,c(2,grep("Warm",colnames(Predict.N.out_Warm))),], start = 1, end = DIM.1, thin = 1  ) ) ,digits) )

# h2.obs contrast
h2.obs.contrast_Temperature = NULL
h2.obs.contrast_Temperature = cbind( "mean" = round( mean( 
	as.mcmc( Predict.N.out_Cold[ , grep("vcv.h2.obs.Cold",colnames(Predict.N.out_Warm)),], start = 1, end = N, thin = 1 ) - as.mcmc( Predict.N.out_Warm[ , grep("vcv.h2.obs.Warm",colnames(Predict.N.out_Cold)),], start = 1, end = N, thin = 1  ) ) ,digits),
	round( HPDinterval( 
	as.mcmc( Predict.N.out_Cold[ , grep("vcv.h2.obs.Cold",colnames(Predict.N.out_Warm)),], start = 1, end = N, thin = 1 ) - as.mcmc( Predict.N.out_Warm[ , grep("vcv.h2.obs.Warm",colnames(Predict.N.out_Cold)),], start = 1, end = N, thin = 1  ) ) ,digits) )
h2.obs.contrast_Temperature

# RG.obs 
RG.obs_Temperature = NULL
RG.obs_Temperature = cbind( "mean" = round( mean( 
	as.mcmc( Predict.N.out_Cold[ , grep("CORgen.obs",colnames(Predict.N.out_Cold)),], start = 1, end = N, thin = 1 )) ,digits),
	round( HPDinterval( 
	as.mcmc( Predict.N.out_Cold[ , grep("CORgen.obs",colnames(Predict.N.out_Cold)),], start = 1, end = N, thin = 1 )) ,digits) )
RG.obs_Temperature

# confirm RG.obs = RG.liab
BV.model.GxE_prior.A_COVARS[BV.model.GxE_prior.A_COVARS$Parameter == "RG.SMO.SMO" & BV.model.GxE_prior.A_COVARS$model.name == "BV.model.GxE_prior.A",]


########################################################################
####
#### Table S2 & Figure S1 - "the general linear animal model to test for feed restriction treatment effects"
####
########################################################################

#### function for LRT test for asreml models
LRT.asreml = function(x, y, DF = 0, cut.rownames = FALSE)  {
	#require ("asreml")
	term = ifelse( length(names(x$gammas)[is.na(match(names(x$gammas) , names(y$gammas) ))]) > 0, names(x$gammas)[is.na(match(names(x$gammas) , names(y$gammas) ))], "TEST")
	df = ifelse(DF == 0, length(x$gammas) - length(y$gammas), DF)
	LRT.chisquare = 2*(x$loglik - y$loglik)
	P.value = 1-pchisq(LRT.chisquare, df)
	Kx = length(x$gammas)
	loglikx = x$loglik
	AICx = -2*loglikx + 2*Kx
	Ky = length(y$gammas)
	logliky = y$loglik
	AICy = -2*logliky + 2*Ky
	LRT = data.frame(
		Term = term,
		LRT.chisquare = round(LRT.chisquare,2), 
		DF = df,
		P.value = round(P.value, 6),
		DAIC = round( (AICx - AICy) ,1),
		warning = if(x$last.message != "LogLikelihood Converged" | y$last.message != "LogLikelihood Converged") "convergence problems!" else "LogLikelihoods converged" )
	if(cut.rownames == TRUE) LRT$Term = substr(as.character(LRT$Term),nchar(as.character(LRT$Term))-25, nchar(as.character(LRT$Term)))
return(LRT)
}

# customised output for coefs of asreml models
coef.nice = function(x, digits= 4){
 	require("asreml")
	coef.nice = data.frame(summary(x, all= TRUE)$coef.fixed)
	coef.nice = coef.nice[!is.na(coef.nice$"std.error"),]
	coef.nice = coef.nice[rev(rownames(coef.nice) ),]
	coef.nice$z.ratio = round(coef.nice$z.ratio, 2)
	coef.nice$solution = round(coef.nice$solution, digits)
	coef.nice$std.error = round(coef.nice$std.error, digits)
	return(coef.nice)
}

# customised output for F-tests of asreml models
wald.nice = function(x){
	require("asreml")
	Wald.nice = wald(x, denDF="default", ssType="conditional", maxiter=1)$Wald		
	Wald.nice = Wald.nice[,c(1,2,4,5,6)]
	Wald.nice[,3] = round(Wald.nice[,3],2)
	Wald.nice[,5] = round(Wald.nice[,5],4)
	Wald.nice$Margin[row.names(Wald.nice) == "(Intercept)"] = "-"
	Wald.nice$sign = ifelse(Wald.nice$Pr < 0.05 & Wald.nice$Pr >= 0.01, "*  ",ifelse(Wald.nice$Pr < 0.01 & Wald.nice$Pr >= 0.001, "** ",ifelse(Wald.nice$Pr < 0.001, "***", "   ")))
	return(Wald.nice)
}

#### models (including model selection)
# additional data required:
"MigrantDataGrowth" = NULL
MigrantDataGrowth = read.table(paste0(DataPath,"MigrantDataGrowth.txt"), sep= "\t", header= TRUE)
dim(MigrantDataGrowth)
# [1] 1326   12

MigrantDataGrowth$Date = as.Date(MigrantDataGrowth$Date)
MigrantDataGrowth$MeasurementPoint = ifelse(MigrantDataGrowth$Date < as.Date("2018-10-08"), "P2", "P3")
MigrantDataGrowth$ln.length = log(MigrantDataGrowth$length.cm)
MigrantDataGrowth$date.integer = as.integer(MigrantDataGrowth$Date)-min(as.integer(MigrantDataGrowth$Date))

# convert MCMCglmm-format A^-1 to asreml-format A^-1
library("MCMCglmm")
"MigrantDataPedigree.ASREML.Ainv" = NULL
MigrantDataPedigree.ASREML.Ainv = sm2asreml(MigrantDataPedigree.MCMCglmm.Ainv)

library("asreml") # code is for asreml version 3

SGR.Model = asreml( 
	fixed= ln.length ~ 1
			+ SMO*Temperature*FeedRestriction*date.integer,
	random= ~ str(~Tank/date.integer, ~us(2):id(Tank))
			+ ped(DNA.ID.at.Tagging),		
	ginverse = list(DNA.ID.at.Tagging = MigrantDataPedigree.ASREML.Ainv),
	rcov = ~ units,
	data= MigrantDataGrowth)
SGR.Model = update(SGR.Model, stepsize=0.5, maxiter=125)
SGR.Model = update(SGR.Model, stepsize=0.1, maxiter=125)
# 2922.8115

# variance heteroscedasticity?
# a) VC
MigrantDataGrowth$Tank.Cold = MigrantDataGrowth$Tank
MigrantDataGrowth$Tank.Warm = MigrantDataGrowth$Tank

SGR.Model_TEMP = asreml( 
	fixed= ln.length ~ 1
			+ SMO*Temperature*FeedRestriction*date.integer,
	random= ~ at(Temperature,"Cold"):str(~Tank.Cold/date.integer, ~us(2):id(Tank)) # test
			+ at(Temperature,"Warm"):str(~Tank.Warm/date.integer, ~us(2):id(Tank)) # test
			+ ped(DNA.ID.at.Tagging),		
	ginverse = list(DNA.ID.at.Tagging = MigrantDataPedigree.ASREML.Ainv),
	rcov = ~ units,
	data= MigrantDataGrowth)
SGR.Model_TEMP = update(SGR.Model_TEMP, stepsize=0.5, maxiter=125)
SGR.Model_TEMP = update(SGR.Model_TEMP, stepsize=0.1, maxiter=125)
# 2923.7941

LRT.asreml(SGR.Model_TEMP, SGR.Model)
# no

# b) VR
SGR.Model_TEMP = asreml( 
	fixed= ln.length ~ 1
			+ SMO*Temperature*FeedRestriction*date.integer,
	random= ~ str(~Tank/date.integer, ~us(2):id(Tank))
			+ ped(DNA.ID.at.Tagging),		
	ginverse = list(DNA.ID.at.Tagging = MigrantDataPedigree.ASREML.Ainv),
	rcov = ~ at(Temperature):units, # test
	data= MigrantDataGrowth)
SGR.Model_TEMP = update(SGR.Model_TEMP, stepsize=0.5, maxiter=125)
SGR.Model_TEMP = update(SGR.Model_TEMP, stepsize=0.1, maxiter=125)
# 2931.7064

LRT.asreml(SGR.Model_TEMP, SGR.Model)
# yes

# c) VA
SGR.Model_TEMP2 = asreml(
	fixed= ln.length ~ 1
			+ SMO*Temperature*FeedRestriction*date.integer,
	random= ~ str(~Tank/date.integer, ~us(2):id(Tank))
			+ us(Temperature):ped(DNA.ID.at.Tagging), # test	
	ginverse = list(DNA.ID.at.Tagging = MigrantDataPedigree.ASREML.Ainv),
	rcov = ~ at(Temperature):units,
	data= MigrantDataGrowth)
SGR.Model_TEMP2 = update(SGR.Model_TEMP2, stepsize=0.5, maxiter=125)
SGR.Model_TEMP2 = update(SGR.Model_TEMP2, stepsize=0.1, maxiter=125)
# 2952.7833

LRT.asreml(SGR.Model_TEMP2, SGR.Model_TEMP)
# yes

wald.nice(SGR.Model_TEMP2)
# keeping all fixed effects

# predict SGR means
MigrantDataGrowth$Temperature = factor(MigrantDataGrowth$Temperature,levels = c("Cold","Warm"))
levels(MigrantDataGrowth$Temperature)
MigrantDataGrowth$SMO = factor(MigrantDataGrowth$SMO,levels = c(0,1))
levels(MigrantDataGrowth$SMO )
diff(range(MigrantDataGrowth$date.integer))

"PREDICT.SGR.Model_TEMP2" = NULL
PREDICT.SGR.Model_TEMP2 = predict(SGR.Model_TEMP2,
	classify = "Temperature:FeedRestriction:SMO:date.integer",
			levels=list(
				SMO = c("0","1"),
				date.integer = c(0,1)), sed=TRUE)$prediction
PREDICT.SGR.Model_TEMP2

"PREDICT.SGR.Model_TEMP2.SGR" = NULL
for(TEMP in 1:2) {
	for(SMO in 1:2) {
		for(RESTR in 1:2) {
		SELECT = which(
			PREDICT.SGR.Model_TEMP2$pvals$Temperature == c("Cold","Warm")[TEMP] &
			PREDICT.SGR.Model_TEMP2$pvals$SMO == c("0","1")[SMO] &
			PREDICT.SGR.Model_TEMP2$pvals$FeedRestriction == c("F","R")[RESTR])
		DATA = PREDICT.SGR.Model_TEMP2$pvals[SELECT,]
		SED = PREDICT.SGR.Model_TEMP2$sed[SELECT,SELECT]
		temp = data.frame(DATA[1,1:3], SGR = round(DATA$predicted.value[DATA$date.integer == 1] - DATA$predicted.value[DATA$date.integer == 0],6), SGR.SE = round(SED[1,2],6))
		PREDICT.SGR.Model_TEMP2.SGR = rbind(PREDICT.SGR.Model_TEMP2.SGR,temp)
}}}
PREDICT.SGR.Model_TEMP2.SGR$lower = round(PREDICT.SGR.Model_TEMP2.SGR$SGR - 2* PREDICT.SGR.Model_TEMP2.SGR$SGR.SE,6)
PREDICT.SGR.Model_TEMP2.SGR$upper = round(PREDICT.SGR.Model_TEMP2.SGR$SGR + 2* PREDICT.SGR.Model_TEMP2.SGR$SGR.SE,6)
PREDICT.SGR.Model_TEMP2.SGR
PREDICT.SGR.Model_TEMP2.SGR$SGR.retrans		= 100*(exp(
	(1+PREDICT.SGR.Model_TEMP2.SGR$SGR)- 
	(1  + attr(scale(log(MigrantDataGrowth$length.cm)),"scaled:center")))-1)
PREDICT.SGR.Model_TEMP2.SGR$lower.retrans	= 100*(exp(
	(1+PREDICT.SGR.Model_TEMP2.SGR$lower)  + attr(scale(log(MigrantDataGrowth$length.cm)),"scaled:center") - 
	(1  + attr(scale(log(MigrantDataGrowth$length.cm)),"scaled:center")))-1)
PREDICT.SGR.Model_TEMP2.SGR$upper.retrans	= 100*(exp(
	(1+PREDICT.SGR.Model_TEMP2.SGR$upper)  + attr(scale(log(MigrantDataGrowth$length.cm)),"scaled:center") - 
	(1  + attr(scale(log(MigrantDataGrowth$length.cm)),"scaled:center")))-1)
PREDICT.SGR.Model_TEMP2.SGR$SGR.SE = NULL
PREDICT.SGR.Model_TEMP2.SGR 

# for plotting means /slopes
PREDICT.SGR.Model_TEMP2 = NULL
PREDICT.SGR.Model_TEMP2 = predict(SGR.Model_TEMP2,
	classify = "Temperature:FeedRestriction:SMO:date.integer",
			levels=list(
				SMO = c("0","1"),
				date.integer = 
					seq(
						min(MigrantDataGrowth$date.integer[MigrantDataGrowth$MeasurementPoint=="P2"]),
						max(MigrantDataGrowth$date.integer[MigrantDataGrowth$MeasurementPoint=="P3"]),
						length= max(MigrantDataGrowth$date.integer[MigrantDataGrowth$MeasurementPoint=="P3"]) - min(MigrantDataGrowth$date.integer[MigrantDataGrowth$MeasurementPoint=="P2"]))
						), sed=TRUE)$prediction$pvals
head(PREDICT.SGR.Model_TEMP2)
PREDICT.SGR.Model_TEMP2$length.cm = exp(PREDICT.SGR.Model_TEMP2$predicted.value)
PREDICT.SGR.Model_TEMP2$lower = exp(PREDICT.SGR.Model_TEMP2$predicted.value - 2*PREDICT.SGR.Model_TEMP2$standard.error)
PREDICT.SGR.Model_TEMP2$upper = exp(PREDICT.SGR.Model_TEMP2$predicted.value + 2*PREDICT.SGR.Model_TEMP2$standard.error)
PREDICT.SGR.Model_TEMP2$Date = as.Date(PREDICT.SGR.Model_TEMP2$date.integer + min(as.integer(MigrantDataGrowth$Date)), origin= "1970-01-01")
PREDICT.SGR.Model_TEMP2

# SGR group means and FeedRestriction contrasts
# SGR RESTR contrasts, cannot extract se of SGR contrasts directly via prediction and se / sed (?)
"PREDICT.SGR.Model_TEMP2.DELTA" = NULL

# TEMP COLD SMO = 0 default
temp = data.frame(Temperature = "Cold", SMO = 0, coef.nice(SGR.Model,digits=6)[rownames(coef.nice(SGR.Model)) == "FeedRestriction_R:date.integer",])
temp
PREDICT.SGR.Model_TEMP2.DELTA = temp

# TEMP COLD SMO = 1
MigrantDataGrowth$Temperature = factor(MigrantDataGrowth$Temperature,levels = c("Cold","Warm"))
levels(MigrantDataGrowth$Temperature)
MigrantDataGrowth$SMO = factor(MigrantDataGrowth$SMO,levels = c(1,0))
levels(MigrantDataGrowth$SMO )

temp = update(SGR.Model_TEMP2,
	fixed= ln.length ~ 1
			+ SMO*Temperature*FeedRestriction*date.integer,
	random= ~ str(~Tank/date.integer, ~us(2):id(Tank))
			+ us(Temperature):ped(DNA.ID.at.Tagging),		
	ginverse = list(DNA.ID.at.Tagging = MigrantDataPedigree.ASREML.Ainv),
	rcov = ~ at(Temperature):units,
	data= MigrantDataGrowth)
temp = update(temp)
temp = update(temp)
temp = data.frame(Temperature = "Cold", SMO = 1, coef.nice(temp,digits=6)[rownames(coef.nice(temp)) == "FeedRestriction_R:date.integer",])
temp
PREDICT.SGR.Model_TEMP2.DELTA = rbind(PREDICT.SGR.Model_TEMP2.DELTA,temp)
PREDICT.SGR.Model_TEMP2.DELTA

# TEMP WARM SMO = 1
MigrantDataGrowth$Temperature = factor(MigrantDataGrowth$Temperature,levels = c("Warm","Cold"))
levels(MigrantDataGrowth$Temperature)
MigrantDataGrowth$SMO = factor(MigrantDataGrowth$SMO,levels = c(1,0))
levels(MigrantDataGrowth$SMO )

temp = update(SGR.Model_TEMP2,
	fixed= ln.length ~ 1
			+ SMO*Temperature*FeedRestriction*date.integer,
	random= ~ str(~Tank/date.integer, ~us(2):id(Tank))
			+ us(Temperature):ped(DNA.ID.at.Tagging),		
	ginverse = list(DNA.ID.at.Tagging = MigrantDataPedigree.ASREML.Ainv),
	rcov = ~ at(Temperature):units,
	data= MigrantDataGrowth)
temp = update(temp)
temp = update(temp)
temp = data.frame(Temperature = "Warm", SMO = 1, coef.nice(temp,digits=6)[rownames(coef.nice(temp)) == "FeedRestriction_R:date.integer",])
temp
PREDICT.SGR.Model_TEMP2.DELTA = rbind(PREDICT.SGR.Model_TEMP2.DELTA,temp)
PREDICT.SGR.Model_TEMP2.DELTA

# TEMP WARM SMO = 0
MigrantDataGrowth$Temperature = factor(MigrantDataGrowth$Temperature,levels = c("Warm","Cold"))
levels(MigrantDataGrowth$Temperature)
MigrantDataGrowth$SMO = factor(MigrantDataGrowth$SMO,levels = c(0,1))
levels(MigrantDataGrowth$SMO )

temp = update(SGR.Model_TEMP2,
	fixed= ln.length ~ 1
			+ SMO*Temperature*FeedRestriction*date.integer,
	random= ~ str(~Tank/date.integer, ~us(2):id(Tank))
			+ us(Temperature):ped(DNA.ID.at.Tagging),		
	ginverse = list(DNA.ID.at.Tagging = MigrantDataPedigree.ASREML.Ainv),
	rcov = ~ at(Temperature):units,
	data= MigrantDataGrowth)
temp = update(temp)
temp = update(temp)
temp = data.frame(Temperature = "Warm", SMO = 0, coef.nice(temp,digits=6)[rownames(coef.nice(temp)) == "FeedRestriction_R:date.integer",])
temp
PREDICT.SGR.Model_TEMP2.DELTA = rbind(PREDICT.SGR.Model_TEMP2.DELTA,temp)
PREDICT.SGR.Model_TEMP2.DELTA$lower = PREDICT.SGR.Model_TEMP2.DELTA$solution - 2*PREDICT.SGR.Model_TEMP2.DELTA$std.error
PREDICT.SGR.Model_TEMP2.DELTA$upper = PREDICT.SGR.Model_TEMP2.DELTA$solution + 2*PREDICT.SGR.Model_TEMP2.DELTA$std.error
rownames(PREDICT.SGR.Model_TEMP2.DELTA) = NULL
PREDICT.SGR.Model_TEMP2.DELTA


########################################################################
####
#### Table S2 - "the generalised linear mixed model to estimate male maturation rate"
####
########################################################################
# subset data for warm and males
"MigrantData_Warm" = NULL
MigrantData_Warm = droplevels(MigrantData[which(MigrantData$Temperature == "Warm"),])
"MigrantData_Warm_Males" = NULL
MigrantData_Warm_Males = droplevels(MigrantData_Warm[which(MigrantData_Warm$Sex == "Male"),])
MigrantData_Warm_Males$DNA.ID.at.Tagging = as.character(MigrantData_Warm_Males$DNA.ID.at.Tagging)
MigrantDataPedigree$DNA.ID.at.Tagging = as.character(MigrantDataPedigree$DNA.ID.at.Tagging)

"MigrantDataPedigree_Warm_Males" = NULL
MigrantDataPedigree_Warm_Males = MigrantDataPedigree
MigrantDataPedigree_Warm_Males$FLAG = "REMOVE"
MigrantDataPedigree_Warm_Males[which( MigrantDataPedigree_Warm_Males$DNA.ID.at.Tagging %in% MigrantData_Warm_Males$DNA.ID.at.Tagging ),]$FLAG = "KEEP"
MigrantDataPedigree_Warm_Males[grep("OUL.", MigrantDataPedigree_Warm_Males$DNA.ID.at.Tagging, fixed=TRUE),]$FLAG = "KEEP"
MigrantDataPedigree_Warm_Males[grep("ML_TAI_17_", MigrantDataPedigree_Warm_Males$DNA.ID.at.Tagging, fixed=TRUE),]$FLAG = "KEEP"
MigrantDataPedigree_Warm_Males = droplevels(MigrantDataPedigree_Warm_Males[which( MigrantDataPedigree_Warm_Males$FLAG != "REMOVE" ),])
MigrantDataPedigree_Warm_Males$FLAG = NULL

"MigrantDataPedigree_Warm_Males.MCMCglmm.Ainv" = NULL
library("MCMCglmm")
MigrantDataPedigree_Warm_Males.MCMCglmm.Ainv = inverseA(MigrantDataPedigree_Warm_Males[,1:3])$Ainv 

# prior
prior.UNI.X21 = NULL
prior.UNI.X21 = list(
	R = list(V = 1, fix= 1), 
	G = list(G1 = list(V= 1, nu= 1000, alpha.mu= 0, alpha.V= 1),
			 G2 = list(V= 1, nu= 1000, alpha.mu= 0, alpha.V= 1))) 
prior.UNI.X21

# settings
SAMPLES = 5000 +10000
BURNIN = 0
THIN = 100 
NITT = BURNIN + THIN*SAMPLES
NITT

# model
library("parallel")
cl = makeCluster(4)

cl.pkg = clusterEvalQ(cl, library("MCMCglmm")) 
clusterExport(cl, "MigrantData_Warm_Males")
clusterExport(cl, "MigrantDataPedigree_Warm_Males.MCMCglmm.Ainv")
clusterExport(cl, "prior.UNI.X21")
clusterExport(cl, "SAMPLES")
clusterExport(cl, "BURNIN")
clusterExport(cl, "THIN")
clusterExport(cl, "NITT")

MAT.UNI_Warm_LIST4 = NULL
MAT.UNI_Warm_LIST4 = parLapply(cl= cl, 1:4, function(i) {
		
	MCMCglmm(
		fixed= MAT ~ 1,
		random= ~ Tank  
				+ DNA.ID.at.Tagging,	
		rcov=   ~ units,
		data= MigrantData_Warm_Males, 
		ginverse= list( DNA.ID.at.Tagging = MigrantDataPedigree_Warm_Males.MCMCglmm.Ainv),
		trunc= TRUE,
		# pr= TRUE,
		family= "threshold",
		prior= prior.UNI.X21 ,
			nitt= NITT,
			burnin= BURNIN,
			thin= THIN)
		
})
stopCluster(cl)

"model.list" = NULL
model.list = MAT.UNI_Warm_LIST4
rm(MAT.UNI_Warm_LIST4)

# subset for Sol posteriors
Sol.list = lapply(model.list, function(x) x$Sol)
Sol.list = do.call(coda:::mcmc.list, Sol.list)
str(Sol.list)

# assess mixing & posterior distributions (Sol), chains overlaid
plot(Sol.list)

rm(Sol.list)

# subset for VCV posteriors
VCV.list = lapply(model.list, function(x) x$VCV[,-which(grepl("units",colnames(x$VCV)))])
VCV.list = do.call(coda:::mcmc.list, VCV.list)
str(VCV.list)

# assess mixing & posterior distributions (VCV), chains overlaid
plot(VCV.list)

# Gelman-Rubin plot
par(ask=TRUE)
gelman.plot(VCV.list, auto.layout=TRUE)

gelman.diag.res = NULL
for(i in 1:ncol(VCV.list[[1]]))
{
gelman.diag.res.temp = data.frame(Term = attr(VCV.list[[1]],"dimnames")[[2]][i])
gelman.diag.res.temp = cbind(gelman.diag.res.temp, gelman.diag(VCV.list[,i])$psrf ) 
gelman.diag.res = rbind( gelman.diag.res , gelman.diag.res.temp )
}
gelman.diag.res

rm(VCV.list)

cor.temp = NULL
for(i in 1:4){
cor.temp = rbind(cor.temp,round(autocorr.diag(model.list[[i]]$VCV)[2,],3))
}
t(cor.temp)

# no need to thin, still thinning
model.list = lapply(model.list, postburnin.MCMCglmm, burnin=250000)
model.list = lapply(model.list, postthin.MCMCglmm, thin=500)

cor.temp = NULL
for(i in 1:4){
cor.temp = rbind(cor.temp,round(autocorr.diag(model.list[[i]]$VCV)[2,],3))
}
t(cor.temp)

# concatenate chains
"MAT.UNI_Warm" = NULL
MAT.UNI_Warm = model.list[[1]]

THIN = thin(MAT.UNI_Warm$VCV) 
THIN
START = start(MAT.UNI_Warm$VCV)
START
END = nrow(MAT.UNI_Warm$VCV) * THIN + START-1
END
END.full = nrow(MAT.UNI_Warm$VCV) *4 * THIN + START-1
END.full

MAT.UNI_Warm$VCV = mcmc(rbind(MAT.UNI_Warm$VCV, model.list[[2]]$VCV, model.list[[3]]$VCV, model.list[[4]]$VCV), start=START, end=END.full, thin=THIN)
MAT.UNI_Warm$Sol = mcmc(rbind(MAT.UNI_Warm$Sol, model.list[[2]]$Sol, model.list[[3]]$Sol, model.list[[4]]$Sol), start=START, end=END.full, thin=THIN)
summary(MAT.UNI_Warm$VCV[,1])

# assess combined chains
round(autocorr.diag(MAT.UNI_Warm$VCV)[2,],3)

plot(MAT.UNI_Warm$Sol[,1:nrow(summary(MAT.UNI_Warm)$solution)])
plot(MAT.UNI_Warm$VCV)

summary(MAT.UNI_Warm)

# predict means on response scale
model = MAT.UNI_Warm

	temp = NULL
	temp = MCMCglmm::predict.MCMCglmm(
		newdata = MigrantData_Warm_Males,
		object= model, 
		marginal= model$Random$formula, 
		type= "response", 
		interval= "confidence")
	str(temp)
	temp = round(data.frame(temp),2)
	names(temp) = c("predicted.value", "lower", "upper")
gc()

predict.MAT.UNI_Warm = unique(data.frame(temp, 
		Temperature = MigrantData_Warm_Males$Temperature,
		Sex = MigrantData_Warm_Males$Sex))
predict.MAT.UNI_Warm

